/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Rook class.  Contains a chess piece representing a rook.
 */

package chess;

import java.util.*;

class Rook extends Piece implements java.io.Serializable {

  int noMoves = 0;  //  Can castle only if it hasn't moved.

  Rook(boolean col){
    value = (col? 500 : -500);
    color = col;
  }

  Vector generateMoves(){
    Vector rookMoves = new Vector(15);
    char c = getColumn();
    int r = getRow();
    Board b = getBoard();
    Piece p;
    // Rook movements
    
                                 // Upward moves 
    for (int ri = r+1; ri <= 8; ri++){
      if ((p = b.getPieceAt(c,ri)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  rookMoves.addElement(new Move(this, c, r, c, ri));
	break;
      } else
	rookMoves.addElement(new Move(this, c, r, c, ri));
    }
	 
                                 // Downward moves 
    for (int ri = r-1; ri >= 1; ri--){
      if ((p = b.getPieceAt(c,ri)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  rookMoves.addElement(new Move(this, c, r, c, ri));
	break;
      } else
	rookMoves.addElement(new Move(this, c, r, c, ri));
    }
	 
                                 // Leftward moves 
    for (char ci = (char) (c-1); ci >= 'a'; ci--){
      if ((p = b.getPieceAt(ci,r)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  rookMoves.addElement(new Move(this, c, r, ci, r));
	break;
      } else
	rookMoves.addElement(new Move(this, c, r, ci, r));
    }

                                 // Rightward moves 
    for (char ci = (char) (c+1); ci <= 'h'; ci++){
      if ((p = b.getPieceAt(ci,r)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  rookMoves.addElement(new Move(this, c, r, ci, r));
	break;
      } else
	rookMoves.addElement(new Move(this, c, r, ci, r));
    }

    return rookMoves;
  }

  boolean isProtected(){
    // Incomplete.
    return true;
  }

  boolean attacksCenter(){
    // Incomplete.
    return false;
  }

  boolean makesCheck(){
    // Incomplete.
    return false;
  }

  void incMoves(){
    noMoves++;
  }

  void decMoves(){
    noMoves--;
  }

  boolean hasMoved(){
    return (noMoves > 0);
  }

  public String toString(){
    return ((color == Game.WHITE)? "R" : "r");
  }

}
